#ifndef _AUDIT_CONFIGURATION_H_
#define _AUDIT_CONFIGURATION_H_

#include "genericConfiguration.h"

class AuditConfiguration : public GenericLogModeConfiguration
{
    Q_OBJECT

public:
    AuditConfiguration();
};

#endif // _AUDIT_CONFIGURATION_H_
